//---------------------------------------------------------------------
//  Copyright (C) Microsoft Corporation.  All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
//---------------------------------------------------------------------

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Windows.Forms.Design.Behavior;

namespace DesignSurfaceExtenderLibrary.Extensions
{
    /// <summary>
    /// An <see cref="DesignSurfaceExtension"/> for managing 
    /// the <see cref="Control.Padding"/> property of all controls
    /// on the design surface.
    /// </summary>
    internal class PaddingExtension : DesignSurfaceExtension
    {
        internal PaddingExtension( IServiceProvider serviceProvider )
            : base( serviceProvider )
        {

        }

        /// <summary>
        /// Creates a <see cref="PaddingGlyph"/> and adds it to the given
        /// adorner's <see cref="Adorner.Glyphs"/> collection.
        /// </summary>
        /// <param name="c">The <see cref="Control"/> which is related to 
        /// the new <see cref="PaddingGlyph"/>.</param>
        protected override void AddGlyphsForControl( Control c )
        {
            if( !( c is Form ) )
            {
                this.adornerValue.Glyphs.Add( new PaddingGlyph(
                    c,
                    base.BehaviorService,
                    base.ComponentChangeService,
                    base.SelectionService,
                    base.ReferenceService,
                    base.ServiceProvider,
                    this.adornerValue ) );
            }
        }

        /// <summary>
        /// Removes the <see cref="PaddingGlyph"/> associated with
        /// the given control from the <see cref="Adorner.Glyphs"/> collection.
        /// </summary>
        /// <param name="c">The <see cref="Control"/> associated with 
        /// the <see cref="PaddingGlyph"/>.</param>
        protected override void RemoveGlyphsForControl( Control c )
        {
            if (c != null)
            {
                for (int i = 0; i < this.adornerValue.Glyphs.Count; i++)
                {
                    PaddingGlyph g = this.adornerValue.Glyphs[i] as PaddingGlyph;

                    if (object.ReferenceEquals(c, g.RelatedControl))
                    {
                        this.adornerValue.Glyphs.Remove(g);
                        break;
                    }
                }
            }
        }

        public override DesignerActionList ActionList
        {
            get
            {
                if( this.actionListValue == null )
                {
                    this.actionListValue = new PaddingActionList( this );
                }

                return this.actionListValue;
            }
        }
    }
}
